@@ -3,6 +3,8 @@
                 Date:           11/7/2001
 				Functions called on actions from the GTK interface
 */
+#define GTK_ENABLE_BROKEN 
+
 #include <glib.h>
 #include <gdk/gdkkeysyms.h>
 #include <ctype.h>
@@ -42,7 +44,7 @@ void callback_button_load (GtkButton* button, gpointer entry)  /* Load button pr
 
  list = gtk_object_get_data (GTK_OBJECT(entry),"list");   /* append filename to list of past filenames*/
  if (!(element = g_list_find_custom(list, filename, (GCompareFunc) g_strcasecmp)))
-                list = g_list_prepend (list, filename);
+              list = g_list_prepend (list, filename);
  else         list = g_list_concat (element, g_list_remove_link(list,element));
  
  gtk_object_set_data (GTK_OBJECT (entry), "list", list);
@@ -52,7 +54,7 @@ void callback_button_load (GtkButton* button, gpointer entry)  /* Load button pr
  callback_global_refresh(); /* refresh all */
 }
 
-void callback_load_isa (GtkMenuItem* menuitem, gpointer newvalue) /* the line width and gran */
+void callback_load_isa (GtkMenuItem* menuitem, gpointer newvalue) 
 {
   view_load_isa = (GList*) newvalue;
 }
@@ -94,35 +96,46 @@ void callback_load_isa_binary(GtkButton* button, gpointer entry)
 
 void callback_load_binary(GtkButton* button, gpointer entry)
 {
- char* filename = g_strdup(gtk_entry_get_text (GTK_ENTRY (entry))); /* Get filename*/
  GList* list;
  GList* element; 
  int file;
  int rep;
  uchar buf[80];
+ Lint lintaddress;
  uchar* address = g_new0(uchar,board_memory_ptr_width);
  struct stat status;
  int sofar = 0;
- char* text = gtk_editable_get_chars(GTK_EDITABLE(view_binary_load_address),0,-1);
+ char* filename;
+ char* text;
+ 
+ text = gtk_editable_get_chars(GTK_EDITABLE(view_binary_load_address),0,-1);
+ 
  if (!view_hexstr2chararr(board_memory_ptr_width, text, address))        /* get address from entry*/
-                {g_free(filename);g_free(text); g_free(address);return;}
+                {g_free(text); g_free(address);return;}
+ lintaddress = lint_from_chararr(board_memory_ptr_width, address);
  g_free(text);
+
+ filename = g_strdup(gtk_entry_get_text (GTK_ENTRY (entry))); /* Get filename*/
+
+ if (!*filename) {g_free(filename); g_free(address);return;}
  file = open(filename, O_RDONLY);
- if (!*filename) {g_free(filename);g_free(text); g_free(address);return;}
+ if (!file) {g_free(filename); g_free(address);return;}
+ 
  gtk_widget_hide(view_fileerror);
  fstat(file, &status);
- while ((rep = read(file, buf,80)))   {
+ while ((rep = read(file, buf, 80))) {
     sofar+=rep;
+    lint_to_chararr(board_memory_ptr_width, address, lintaddress);
     board_set_memory (rep, address, buf, 1);
     if (!board_micro_ping())   {
             board_set_memory (rep, address, buf, 1);
-            if (!board_micro_ping()) {g_free(filename);g_free(text); g_free(address);return;}
+            if (!board_micro_ping()) {g_free(filename); g_free(address);return;}
             }                    /* if errored with microping then do again*/
-    view_chararrAddintsame(board_memory_ptr_width, address, rep); /* send data 80 at a time*/
-    
+    lintaddress = lint_operate(lintaddress, lint_new(rep), LINT_ADD);           /* send data 80 at a time*/
     callback_main_progress_bar_update((float)sofar/status.st_size);/* update progress bar*/
     while (gtk_events_pending())  gtk_main_iteration();          /*update display and do user stuff*/
     }
+ 
  callback_main_progress_bar_update(1);/* update progress bar*/
  g_free(address);
  close (file);
@@ -132,7 +145,7 @@ void callback_load_binary(GtkButton* button, gpointer entry)
    else         list = g_list_concat (element, g_list_remove_link(list,element));
    gtk_object_set_data (GTK_OBJECT (entry), "list", list);
    gtk_combo_set_popdown_strings (GTK_COMBO (GTK_WIDGET(entry)->parent), list);
-  gtk_entry_set_text (GTK_ENTRY(entry), filename);
+   gtk_entry_set_text (GTK_ENTRY(entry), filename);
  if (element) g_free(filename);
  callback_memory_refresh(); /* refresh all */
 }
@@ -592,20 +605,24 @@ void callback_step_number(GtkEditable *editable, gpointer value)
 }
 
 
-void callback_memwindow_length (GtkMenuItem* menuitem, gpointer newvalue) /* the line width and gran */
+void callback_memwindow_length (GtkMenuItem* menuitem, gpointer newvalue) 
 {
   int temp;
+  int columns;
   mem_window* memwin = view_getmemwindowptr(GTK_WIDGET (menuitem));
-  memwin->width = (int) newvalue&15;                  /* the newvalue holds the granularty and width */
-  memwin->gran = (int) newvalue/16;
+  memwin->width = ((mem_window_length*) newvalue)->width;                  /* the newvalue holds the granularty and width */
+  memwin->gran =  ((mem_window_length*) newvalue)->gran;
+  if (memwin->gran) columns = memwin->width/memwin->gran;
+  else columns=1;
+  
   gtk_clist_freeze (GTK_CLIST(memwin->clist_ptr)); 
 
-  for (temp=4;temp>(memwin->width/memwin->gran);temp--)
+  for (temp=4;temp>columns;temp--)
     gtk_clist_set_column_visibility(GTK_CLIST(memwin->clist_ptr),temp, FALSE);
   while(temp){
     gtk_clist_set_column_visibility(GTK_CLIST(memwin->clist_ptr),temp,TRUE);/* make hex collumns visible or not*/
     gtk_clist_set_column_width (GTK_CLIST (memwin->clist_ptr), temp,
-                                      (memwin->width*25)/(memwin->width/memwin->gran)-7);
+                                      (memwin->width*25)/(columns)-7);
     temp--;
     }
     gtk_widget_set_usize (memwin->hex_entry, (memwin->width*25), -2);
@@ -635,6 +652,9 @@ void callback_memwindow_listsize (GtkEditable *editable, gpointer ignore) /* cha
   int new = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (editable));
   mem_window* memwin = view_getmemwindowptr(GTK_WIDGET (editable));
   
+  g_free (memwin->offsets);
+  memwin->offsets=g_new0(int, new);
+  
   gtk_clist_freeze (GTK_CLIST(memwin->clist_ptr)); 
   if (memwin->count<new)
     while (memwin->count<new)
@@ -721,40 +741,41 @@ void callback_memwindow_hex (GtkEditable* editable, gpointer ignore) /* return p
   mem_window* memwin = view_getmemwindowptr(GTK_WIDGET (editable));
   char* text = gtk_editable_get_chars(GTK_EDITABLE(memwin->address_entry),0,-1);
   uchar* hex;
-  int row;
   uchar* address = g_new(uchar,board_memory_ptr_width);
-  uchar* new_address;
 
   if (!view_hexstr2chararr(board_memory_ptr_width, text, address))  /* get address from entry*/
                 {g_free(text); g_free(address);return;}
-  if (board_wordaline) address[0] &= -(memwin->gran); /* word aline */
+  if (board_wordaline & memwin->gran) address[0] &= -(memwin->gran); /* word aline */
 
   g_free(text);
-  hex = g_new(uchar,memwin->gran);
+  hex = g_new(unsigned char,memwin->gran ? memwin->gran : memwin->width);
   text = gtk_editable_get_chars(GTK_EDITABLE(memwin->hex_entry),0,-1); /* get hex value*/
-  if (!view_hexstr2chararr(memwin->gran, text, hex))
+  if (!view_hexstr2chararr(memwin->gran ? memwin->gran : memwin->width, text, hex)) // FIXME is it memwin->width or what?
                 {g_free(text); g_free(hex); g_free(address);return;}
   g_free(text);
-  board_set_memory(1,address,hex,memwin->gran);      /* set memory*/
+  board_set_memory(1,address,hex,memwin->gran ? memwin->gran : memwin->width);      /* set memory*/
   g_free(hex);
-  new_address = view_chararrAddchararr(board_memory_ptr_width,
-                         memwin->address, memwin->regoffset);
-  row = view_chararrSubchararr(board_memory_ptr_width, address, new_address)
-                                                              + memwin->gran; /* move row one forard*/
-  g_free(new_address);
-  g_free(address);
-  if (row == memwin->count * memwin->width){  /* if at end then move list one foward*/
-  view_chararrAddintsame(board_memory_ptr_width, memwin->address, memwin->width);
-  row -= memwin->width;
-  }
-  if (row < memwin->count * memwin->width)
-        gtk_clist_select_row(GTK_CLIST(memwin->clist_ptr),row/memwin->width
-                                       ,(row%memwin->width)/memwin->gran+1);
+
+
+
+//  new_address = view_chararrAddchararr(board_memory_ptr_width,
+//                         memwin->address, memwin->regoffset);
+//  row = view_chararrSubchararr(board_memory_ptr_width, address, new_address)
+//                                                              + memwin->gran; /* move row one forard*/
+//  g_free(new_address);
+    g_free(address);
+//  if (row == memwin->count * memwin->width){  /* if at end then move list one foward*/
+//  view_chararrAddintsame(board_memory_ptr_width, memwin->address, memwin->width);
+//  row -= memwin->width;
+//  }
+//  if (row < memwin->count * memwin->width)
+//        gtk_clist_select_row(GTK_CLIST(memwin->clist_ptr),row/memwin->width
+//                                       ,(row%memwin->width)/memwin->gran+1);
                                     
  
   callback_memory_refresh(); /* refresh all */
   while (gtk_events_pending())  gtk_main_iteration();                /*update display */
-  gtk_entry_select_region (GTK_ENTRY(memwin->hex_entry),0,-1); /* select all of region*/
+  gtk_entry_select_region (GTK_ENTRY(memwin->hex_entry),0,-1); /* select all of hex region*/
 }
 
 
@@ -782,7 +803,7 @@ void callback_memwindow_dis (GtkEditable* editable, gpointer ignore)
   
   if (!view_hexstr2chararr(board_memory_ptr_width, text, address))  /* get address from entry*/
                 {g_free(text); g_free(address);return;}
-  if (board_wordaline) address[0] &= -(memwin->gran); /* word aline */
+  if (board_wordaline && memwin->gran) address[0] &= -(memwin->gran); /* word aline */
 
   g_free(text);
   
@@ -802,13 +823,13 @@ void callback_memwindow_dis (GtkEditable* editable, gpointer ignore)
     while(symbol_list) {
         symbol = symbol_findstring(symbol_table, (char*)symbol_list->data);
         if (symbol) {
-            symbol->value = lint_new(view_chararr2int(board_memory_ptr_width, address));
+            symbol->value = lint_from_chararr(board_memory_ptr_width, address);
             }
         else {
             symbol = g_new(Symbol, 1);
             symbol->flags = 2;
             symbol->name = symbol_list->data;
-            symbol->value = lint_new((long long)view_chararr2int(board_memory_ptr_width, address));
+            symbol->value = lint_from_chararr(board_memory_ptr_width, address);
             symbol_table = g_list_append(symbol_table,symbol);
 //            g_print("%x:\"%s\"\n", address, symbol->name);
             }
@@ -826,20 +847,22 @@ void callback_memwindow_dis (GtkEditable* editable, gpointer ignore)
     
   board_set_memory(temp,address,hex,1);      /* set memory*/
   g_free(hex);
+  
   new_address = view_chararrAddchararr(board_memory_ptr_width,
                          memwin->address, memwin->regoffset);
   row = view_chararrSubchararr(board_memory_ptr_width, address, new_address)
                                                               + temp; /* move row one forard*/
 
-  g_free(new_address);
-  g_free(address);
-  if (row == memwin->count * memwin->width){  /* if at end then move list one foward*/
-  view_chararrAddintsame(board_memory_ptr_width, memwin->address, memwin->width);
-  row -= memwin->width;
-  }
-  if (row < memwin->count * memwin->width)
-      gtk_clist_select_row(GTK_CLIST(memwin->clist_ptr),row/memwin->width
-                                       ,(row%memwin->width)/memwin->gran+1);
+// g_free(new_address);
+// g_free(address);
+// if (row == memwin->count * memwin->width){  /* if at end then move list one foward*/
+// view_chararrAddintsame(board_memory_ptr_width, memwin->address, memwin->width);
+// row -= memwin->width;
+// }
+// if (row < memwin->count * memwin->width)
+//     gtk_clist_select_row(GTK_CLIST(memwin->clist_ptr),row/memwin->width
+//                                      ,(row%memwin->width)/memwin->gran+1);
+
   gtk_editable_select_region (GTK_EDITABLE(memwin->dis_entry),0,-1); /* select all of region*/
   if (gtk_events_pending()>2)
       while (gtk_events_pending())  gtk_main_iteration();                /*update display */
@@ -898,10 +921,12 @@ void  callback_memwindow_clist (GtkCList *clist, gint row, gint column,  /* a li
   uchar* address;
   char* address_text;
   uchar* memdata;
+  int consumed;
   
-  if (board_wordaline) new_address[0] &= -memwin->width;              /* word aline   */
+  if (board_wordaline && memwin->gran) new_address[0] &= -memwin->width;              /* word aline   */
+
   address = view_chararrAddint(board_memory_ptr_width,
-           new_address, row * memwin->width + 
+           new_address, memwin->offsets[row] + 
 		  ((column > 4) ? 0 : (column <= 0) ? 0 : column-1) * memwin->gran);   /* work out wich collumn  */
   g_free(new_address);
   
@@ -959,27 +984,28 @@ void  callback_memwindow_clist (GtkCList *clist, gint row, gint column,  /* a li
     return;
     }
 
-  memdata = g_new(uchar,memwin->gran);
+  memdata = g_new(uchar,memwin->gran ? memwin->gran : 10);
   address_text = view_chararr2hexstrbe(board_memory_ptr_width,address);
   gtk_entry_set_text(GTK_ENTRY(memwin->address_entry),address_text);
   g_free(address_text);
-  if  (!board_get_memory (1, address, memdata, memwin->gran)){    /* get value into the text entry*/
+  
+  if  (!board_get_memory (memwin->gran ? 1 : 10, address, memdata, memwin->gran ? memwin->gran : 1)){    /* get value into the text entry*/
    if (!board_mini_ping(0) ||
-       !board_get_memory (1, address, memdata, memwin->gran)){ /* if that didnt work then dont panic just try again*/
+       !board_get_memory (memwin->gran ? 1 : 10, address, memdata, memwin->gran ? memwin->gran : 1)){ /* if that didnt work then dont panic just try again*/
  	                            g_free(memdata);g_free(address);return;}} /* sorry but board is lost*/
   
-  address_text = view_chararr2hexstrbe (memwin->gran,memdata);  /* set hex entry*/
+  address_text = view_dis(address, memdata, memwin->gran ? memwin->gran : 10, memwin->isa, &consumed);  /* set dis entry */
+  gtk_entry_set_text(GTK_ENTRY(memwin->dis_entry),address_text);  
+  g_free(address_text);
+  
+  address_text = view_chararr2hexstrbe (consumed,memdata);  /* set hex entry*/
   gtk_entry_set_text(GTK_ENTRY(memwin->hex_entry),address_text);
   g_free(address_text); 
   
-  address_text = view_chararr2asciistr (memwin->gran,memdata);  /* set ascii entry */
+  address_text = view_chararr2asciistr (consumed,memdata);  /* set ascii entry */
   gtk_entry_set_text(GTK_ENTRY(memwin->ascii_entry),address_text);  
   g_free(address_text);
   
-  address_text = view_dis(address, memdata, memwin->gran, memwin->isa );  /* set dis entry */
-  gtk_entry_set_text(GTK_ENTRY(memwin->dis_entry),address_text);  
-  g_free(address_text);
-  
   g_free(address);
   g_free(memdata);
   
@@ -1027,7 +1053,7 @@ void  callback_regwindow_clist (GtkCList *clist, gint row, gint column, /* c-lis
        else                                                 value = "0";
   
   gtk_entry_set_text(GTK_ENTRY(regwin->address_entry),
-                     board_reg_banks[regwin->regbank_no].names[row]);
+                     board_reg_banks[regwin->regbank_no].names[row]->data);
   gtk_entry_set_text(GTK_ENTRY(regwin->hex_entry),value);
   if (board_reg_banks[regwin->regbank_no].width) g_free(value);
 }
@@ -1102,7 +1128,7 @@ gint callback_update_comms(gpointer comms)
 gint callback_terminal_keysnoop(GtkWidget *grab_widget, GdkEventKey *event,
                                 gpointer comms)
 {
- uint key_pressed;  /* to save the value of the key that invoked this function */
+ unsigned int key_pressed;  /* to save the value of the key that invoked this function */
  feature *comms_terminal = comms;   /* record feature from which callback came */
 
 
@@ -1352,8 +1378,8 @@ void callback_main_quit(gpointer ignore, gpointer ignore2)
     ScanPrintSCANNode(fd, topnode);
     fclose(fd);
     }
-        
  }
+ ScanDeleteSCANNode(topnode);
 }
  exit(0);
 
