@@ -10,18 +10,18 @@
 #include "chump.h"
 #include "dotparse.h"
 #include "lint.h"
+#include "symboltable.h"
 #include <glib.h>
 #include <string.h>
 #include <ctype.h>
 
-#define myint64 unsigned long long
-
 char* dis_error_string="ERROR";
 const AsmStatus asm_default_status = {NULL, 0, 0, 30};
           // Symbol* symbol_table, long address, int flags;
 
 
 
+
 DefinitionStack *asm_parser (GList * list)          // Pass the parsed list to tis function with the name as the first element
 {                                                   // isa     "ARM32"  (define "reg" ....
 	if (!SCAN_IS_STRING (SCAN_LIST_FIRST (list))){   //      ^ start
@@ -74,21 +74,35 @@ DefinitionStack *asm_define (GList * parselist, DefinitionStack * deflist, char
 				      SCAN_IS_NUMBER ((PtrSCANNode) sublist->next->data) &&
 				      !g_strcasecmp (((PtrSCANNode) sublist->data)->body.string, ENUMERATESTRING)) {
                 int size, number, temp;
+                GList* aliaslist=NULL;
+                char* newstring;
                 Rule *rule;
                 RuleElement *element;
                 number=0;
                 sublist = sublist->next;
                 size = SCAN_NUMBER(SCAN_LIST_FIRST(sublist));
-                while ((sublist = sublist->next)){
-                    if (!SCAN_IS_STRING(SCAN_LIST_FIRST(sublist))){
+                sublist = sublist->next;
+                while (sublist){
+                    if (SCAN_IS_LIST(SCAN_LIST_FIRST(sublist)) && !aliaslist)
+                        aliaslist = SCAN_LIST(SCAN_LIST_FIRST(sublist));
+                    if (!SCAN_IS_STRING(SCAN_LIST_FIRST(sublist)) && !SCAN_IS_LIST(SCAN_LIST_FIRST(sublist))){
                         g_print ("line:%d position:%d  Enum lists should be created using only \"strings\" \n",
 					                    ((PtrSCANNode) sublist->data)->line, ((PtrSCANNode) sublist->data)->position);
                         break;
                         }
+                    if (aliaslist)  {
+                        newstring = SCAN_STRING(SCAN_LIST_FIRST(aliaslist));
+                        }
+
+                    else {
+                        newstring = SCAN_STRING(SCAN_LIST_FIRST(sublist));
+                        }
+                        
+                    
                     rule = g_new(Rule,1);
                     rule->type = TRANRULE;
                     element = g_new(RuleElement,1);
-                    element->string = SCAN_STRING(SCAN_LIST_FIRST(sublist));
+                    element->string = newstring;
                     element->type = TERM;
                     element->twin=0;
                     element->twinsplit=0;
@@ -112,7 +126,13 @@ DefinitionStack *asm_define (GList * parselist, DefinitionStack * deflist, char
                     if (definitionelement->rules->data)
         				    definitionelement->rules = g_list_append (definitionelement->rules, rule);      // add it to the rules
                     else    definitionelement->rules->data = rule;                                          // replace dummy list
-                    number++;
+
+                    if (aliaslist) 
+                        aliaslist = aliaslist->next;
+                    if (!aliaslist) {
+                        sublist = sublist->next;
+                        number++;
+                        }
                     }
                 }
                         
@@ -193,6 +213,18 @@ Rule *asm_rule (GList * parselist, DefinitionStack * deflist)
 		if (!g_strcasecmp (((PtrSCANNode) parselist->data)->body.string, COMMENTSTRING)){
 			rule->type = COMMENTRULE;
             return rule;                                                                    // Its a comment definition but there are other fields to use
+            }
+		if (!g_strcasecmp (((PtrSCANNode) parselist->data)->body.string, ALIGNSTRING)){
+			rule->type = ALIGNRULE;
+            parselist = parselist->next;
+            rule->size = 0;
+            if (!SCAN_IS_NUMBER ((PtrSCANNode) parselist->data)) {
+			    g_print ("line:%d position:%d  ALIGN rules take a number as a paramiter\n",
+				    ((PtrSCANNode) parselist->data)->line, ((PtrSCANNode) parselist->data)->position);
+                return rule;
+                }
+            rule->size = ((PtrSCANNode) parselist->data)->body.number;
+            return rule;                                                                    // Its a align definition but there are other fields to use
             }
 		if (!g_strcasecmp (((PtrSCANNode) parselist->data)->body.string, ASCIISTRING)){
 			rule->type = ASCIIRULE;
@@ -313,7 +345,7 @@ Rule *asm_rule (GList * parselist, DefinitionStack * deflist)
             
  			if (SCAN_IS_NUMBER (SCAN_LIST_FIRST(SCAN_LIST(SCAN_LIST_FIRST(text)))))   // ensure the definition is in quotes, else panic!!!
 			{
-				g_print ("line:%d position:%d  A rule sould start with a string and not a number\n",
+				g_print ("line:%d position:%d  A rule should start with a string and not a number\n",
 					((PtrSCANNode) text->data)->line,
 					((PtrSCANNode) text->data)->position);
 				return rule;
@@ -497,116 +529,115 @@ Rule *asm_rule (GList * parselist, DefinitionStack * deflist)
 		}
 		binary = binary->next; // next!
 	}
-	return rule;        //when donr return rule
+	return rule;        //when done return rule
 }
 
 
 
 
 Disasm asm_disassemble_number (char* bitfield, GList * rules, AsmStatus stat)
-//                  list of binary, list of rules, address (should be bigger. fixme)
+//                  list of binary, list of rules, status
 {
  Disasm ret;
  Disasm error;
  char* binary;
  char *c;
+ int bitcount;
+ int flag = 0;
  RuleElement *newelement;
- myint64 numbera;
- myint64 size;
+ Lint numbera;
+ Lint size;
  Symbol* tempsymbol;
     
  error.distext = &dis_error;         // error dissasembly
  error.bitfield = dis_error_string;
  ret.bitfield = bitfield;
  
-
-
-
-
- if ((int) ((Rule *) rules->data)->bit->data > strlen(bitfield)) return error;  // if its longer than the numbera
- binary =  &ret.bitfield[(int)((Rule *) rules->data)->bit->data - 1];        // of bits fetched then error out
- numbera = 0;                                                   // walk through the binary backwards
- 
- if (asm_isset(*binary)){                        // if its signed then the first bit means the sign
+ bitcount = (int)((Rule *) rules->data)->bit->data;
+ if (bitcount > strlen(bitfield)) return error;                         // if its longer than the numbera
+ binary =  &ret.bitfield[bitcount - 1];                                 // of bits fetched then error out
+                                                                        // walk through the binary backwards
+ if (asm_isset(*binary)){                                               // if its signed then the first bit means the sign
     if (    UINTRULE      == ((Rule *) rules->data)->type
-         || URELATIVERULE == ((Rule *) rules->data)->type)  numbera=1;
-    else                                                    numbera=-1; // numbera is the accumulator
+         || URELATIVERULE == ((Rule *) rules->data)->type)  numbera = lint_new( 1);
+    else                                                    numbera = lint_new(-1); // numbera is the accumulator
     }
+ else                                                       numbera = lint_new( 0);
     
- while (binary != ret.bitfield) {   // walk backwards
+ while (binary != ret.bitfield) {                                           // walk backwards
     binary--;
-    numbera = numbera << 1;
-    if (asm_isset(*binary)) numbera++;   // keep adding and shifting
-	}
+    numbera = lint_operate(numbera, lint_new(1), LINT_SL);
+    if (asm_isset(*binary)) numbera = lint_operate(numbera, lint_new(1), LINT_OR);    // keep orring and shifting
+	}                                                                               // create a lint number from a binary string
 
  {
-    GList *number = g_list_last (((Rule *) rules->data)->bit);      // apply operations from end 
+    GList *number = g_list_last (((Rule *) rules->data)->bit);          // apply operations from end 
     GList *opp    = g_list_last (((Rule *) rules->data)->asci);
-    myint64 imm;
-    while(opp){                                 // all opertions
-        imm=GPOINTER_TO_INT (number->data);     // imm is the operand
-        c= (char*)opp->data;                    // c points to the operator
-        switch (*c){							// if not inverted then just do normal inverse operations
+    Lint imm;
+    while(opp){                                         // all opertions
+        imm = lint_new(GPOINTER_TO_INT(number->data));  // imm is the operand
+        c = (char*)opp->data;                           // c points to the operator
+        switch (*c){							        // if not inverted then just do normal inverse operations
             case '+':
-                numbera -=imm;
-                if (UINTRULE      == ((Rule *) rules->data)->type)
-                    if ((myint64)numbera<0) return error;
+                numbera = lint_operate(numbera, imm, LINT_SUB);
                 break;
                     
             case '~':
-                numbera = imm - numbera;
+                numbera = lint_operate(imm, numbera, LINT_SUB);
                 break;
 
             case '-':
-                numbera +=imm;
+                numbera = lint_operate(numbera, imm, LINT_ADD);
                 break;
             
             case '*':
-                numbera /=imm;
+                numbera = lint_operate(numbera, imm, LINT_DIV);
                 break;
                     
             case '/':
-                numbera *=imm;
+                numbera = lint_operate(numbera, imm, LINT_MUL);
                 break;
             
             case '>':
-                numbera = numbera << imm;
+                numbera = lint_operate(numbera, imm, LINT_SL);
                 break;
 
             case '<':
-                numbera = numbera >> imm;
+                numbera = lint_operate(numbera, imm, LINT_SR);
                 break;
                                 
             case '@':
                 c++;
-                size = g_strtod(c,NULL);
-                numbera = (((numbera >> imm) | (numbera << (size - imm))) & ((((1 << (size-1) )-1)<<1)+1 ));
+                size = lint_from_string(c,&c);
+                if (lint_compare(numbera,lint_operate (lint_new(1),  size,   LINT_SL)) >= 0) {flag = 1;break;}          // FIXME rotating negative numbers?
+                numbera = lint_operate (lint_operate (numbera, imm, LINT_SR), lint_operate (numbera, lint_operate(size, imm, LINT_SUB), LINT_SL), LINT_OR);
+                numbera = lint_operate (numbera, lint_operate (lint_operate (lint_new(1), size, LINT_SL), lint_new(1), LINT_SUB), LINT_AND);
                 break;
 
             default:
                 break;
             }
-							
+        if (flag) break;
         opp=opp->prev;              // next operation!
         number=number->prev;
         }
  }
- if (    (RELATIVERULE == ((Rule *) rules->data)->type)      // if its a relative rule then add the address
-     || (URELATIVERULE == ((Rule *) rules->data)->type)) numbera += stat.address;
 
- ret.bitfield = &ret.bitfield[(int)((Rule *) rules->data)->bit->data]; // advance bitfield forward
+
+ if (    (RELATIVERULE == ((Rule *) rules->data)->type)     // if its a relative rule then add the address
+     || (URELATIVERULE == ((Rule *) rules->data)->type)) numbera = lint_operate(numbera, lint_new(stat.address), LINT_ADD);
+
+ ret.bitfield = &ret.bitfield[bitcount];                    // advance bitfield forward
  
+ newelement = g_new (RuleElement, 1);
  
- newelement = g_new (RuleElement, 1); 
- 
- 
- tempsymbol = symbol_findaddress (stat.symbol_table, lint_new(numbera));
+ tempsymbol = symbol_findaddress (stat.symbol_table, numbera);
  if (tempsymbol) c = tempsymbol->name;
  else            c = NULL;
- if (c && (    (RELATIVERULE == ((Rule *) rules->data)->type)     
+ if (c && (    (RELATIVERULE == ((Rule *) rules->data)->type)
            || (URELATIVERULE == ((Rule *) rules->data)->type)))
         newelement->string = g_strdup(c);
- else   newelement->string = g_strdup_printf ("0x%X", numbera);    // make a new element which will be freed later
+ else   newelement->string = g_strdup_printf ("0x%lX", lint_to_longlong(numbera));    // make a new element which will be freed later
  
  newelement->type = TERM2FREE;                           // and sick our number into it
  ret.distext = g_list_append (NULL, newelement);         // add it to the return text text
@@ -616,7 +647,7 @@ Disasm asm_disassemble_number (char* bitfield, GList * rules, AsmStatus stat)
 
 
 Disasm asm_disassemble_label (char* bitfield, GList * rules, AsmStatus stat)
-//                  list of binary, list of rules, address (should be bigger. fixme)
+//                  list of binary, list of rules, status
 {
  Disasm ret;
  RuleElement *newelement;
@@ -636,7 +667,7 @@ Disasm asm_disassemble_label (char* bitfield, GList * rules, AsmStatus stat)
 
 
 Disasm asm_disassemble_ascii (char* bitfield, GList * rules, AsmStatus stat)
-//                  list of binary, list of rules, address (should be bigger. fixme)
+//                  list of binary, list of rules, status
 {
  Disasm ret;
  RuleElement *newelement;
@@ -705,7 +736,7 @@ Disasm asm_disassemble_ascii (char* bitfield, GList * rules, AsmStatus stat)
 
 
 Disasm asm_disassemble (char* bitfield, GList* rules, AsmStatus stat)
-//                  list of binary, list of rules, address (should be bigger. fixme)
+//                  list of binary, list of rules, status
 {
 	Disasm ret;
 	Disasm rep;
@@ -764,7 +795,7 @@ Disasm asm_disassemble (char* bitfield, GList* rules, AsmStatus stat)
                     g_list_nth (rule->split, ((RuleElement *) binary->data)->twinsplit)->data = newsplit;
                     newsplit->bitfield =  g_strnfill(newsplit->size, '?');
                     for(size=0; size < newsplit->size; size++){
-                        if (!ret.bitfield) {flag=1; break;}
+                        if (!*ret.bitfield) {flag=1; break;}
                         newsplit->bitfield[size] = *ret.bitfield;
         	            ret.bitfield++;
                         }
@@ -780,7 +811,6 @@ Disasm asm_disassemble (char* bitfield, GList* rules, AsmStatus stat)
                             oldsplitbitfield = splitbitfield; 
                             splitbitfield = g_strconcat(((SplitElement*)splitlist->data)->bitfield, splitbitfield, NULL);
                             g_free(oldsplitbitfield);
-                            ((SplitElement*)splitlist->data)->bitfield=NULL;
                             splitlist=splitlist->next;
                             }
                         rep = asm_disassemble (splitbitfield, ((RuleElement *) binary->data)->rules, stat);   // in that case disassable it
@@ -842,6 +872,13 @@ Disasm asm_disassemble (char* bitfield, GList* rules, AsmStatus stat)
        	return ret;
         break;
         
+       case ALIGNRULE:
+        if (stat.address & (((Rule *) rules->data)->size - 1)) break;
+ 		ret.bitfield = bitfield;       
+		ret.distext = NULL;
+       	return ret;
+        break;
+        
        case ASCIIRULE:
         rep = asm_disassemble_ascii (bitfield, rules, stat);
   	    if (dis_error_string != rep.bitfield)      // if replay was OK then return
@@ -867,9 +904,9 @@ Asm* asm_assemble_number (char* text, GList* rules, AsmStatus stat, int last)
  Asm *ret = g_new(Asm,1);
  RuleElement *newelement;
  int flag;
- unsigned int size;
- myint64 value;
- myint64 max;
+ Lint size;
+ unsigned int bitcount;
+ Lint value;
 // int index;
  char* start;
  char* c;
@@ -878,80 +915,90 @@ Asm* asm_assemble_number (char* text, GList* rules, AsmStatus stat, int last)
  flag=0;
 
 
+ bitcount = (unsigned int) ((Rule *) rules->data)->bit->data;
+
  if (stat.flags & ASMFLAG_IGNORESYMBOLS)
-    value = lint_to_longlong(exp_exp(ret->text, &(ret->text), 0, symbol_extract_fake, stat.symbol_table));
+    value = exp_exp(ret->text, &(ret->text), 0, symbol_extract_fake, stat.symbol_table);
  else
-    value = lint_to_longlong(exp_exp(ret->text, &(ret->text), 0, symbol_extract,      stat.symbol_table));
+    value = exp_exp(ret->text, &(ret->text), 0, symbol_extract,      stat.symbol_table);
  if (!ret->text) {g_free(ret); return NULL;}
- max = (myint64) 1<<((unsigned int) ((Rule *) rules->data)->bit->data);
-
-
 
 
  if (    (RELATIVERULE == ((Rule *) rules->data)->type)
-     || (URELATIVERULE == ((Rule *) rules->data)->type)) value -= stat.address; // if relative subtract address
+     || (URELATIVERULE == ((Rule *) rules->data)->type)) value = lint_operate (value, lint_new(stat.address), LINT_SUB); // if relative subtract address
 
  {
     GList *number = ((Rule *) rules->data)->bit->next;  // walk through the operations
     GList *opp = ((Rule *) rules->data)->asci;
-    myint64 imm;
+    Lint imm;
     while(opp){
-        imm=GPOINTER_TO_INT (number->data);
-        c= (char*)opp->data;
+        imm=lint_new(GPOINTER_TO_INT (number->data));
+        c = (char*)opp->data;
         switch (*c){
             case '~':
-                value = imm - value;
+                value = lint_operate (imm, value, LINT_SUB);
                 break;
 
             case '+':
-                value +=imm;
+                value = lint_operate (value, imm, LINT_ADD);
                 break;
     
             case '-':
-                value -=imm;
+                value = lint_operate (value, imm, LINT_SUB);
                 break;
             
             case '*':
-                value *=imm;
+                value = lint_operate (value, imm, LINT_MUL);
                 break;
     
             case '/':
-                value /=imm;
+                if (lint_compare(lint_operate(value, imm, LINT_MOD), lint_new(0))) {flag = 1;break;};
+                value = lint_operate (value, imm, LINT_DIV);
                 break;
             
             case '>':
-                value = value >> imm;
+                if (lint_test(value, lint_operate(lint_operate(lint_new(1), imm, LINT_SL),lint_new(1), LINT_SUB))){flag = 1;break;}
+                if (lint_compare(lint_operate(value,lint_operate (lint_new(1),  imm,   LINT_SL), LINT_OR), lint_new(0)))
+                value = lint_operate (value, imm, LINT_SR);
                 break;
                 
             case '<':
-                value = value << imm;
+                value = lint_operate (value, imm, LINT_SL);
                 break;
                 
             case '@':
                 c++;
-                size = g_strtod(c,NULL);
-                value = ((value << imm) | ((value >> (size - imm)) & ((((1 << (size-1) )-1)<<1)+1 )));
+                size = lint_from_string(c,&c);
+                if (lint_compare(value,lint_operate (lint_new(1),  size,   LINT_SL)) >= 0) {flag = 1;break;}
+                value = lint_operate (lint_operate (value, imm, LINT_SL), lint_operate (value, lint_operate(size, imm, LINT_SUB), LINT_SR), LINT_OR);
+                value = lint_operate (value, lint_operate (lint_operate (lint_new(1), size, LINT_SL), lint_new(1), LINT_SUB), LINT_AND);
                 break;
                 
             default:
                 break;
             }
+        if (flag) break;
         opp=opp->next;
         number=number->next;
         }
  }
 
- if ((value < max) ||
-                   (RELATIVERULE == ((Rule *) rules->data)->type) ||
-                        (INTRULE == ((Rule *) rules->data)->type))
-    if (!flag) {											            // flag is set if something went wrong
-        start = g_new(char,(int) ((Rule *) rules->data)->bit->data + 1);//make a new string
-        c = start + (int) ((Rule *) rules->data)->bit->data;			//terminate it with a /0
+ if ((RELATIVERULE == ((Rule *) rules->data)->type) ||
+          (INTRULE == ((Rule *) rules->data)->type))
+      { if (lint_compare(value,lint_operate (lint_new(1),  lint_new(bitcount-1), LINT_SL)) >= 0) flag = 1;  //TESTME ensure the ranges are correct   
+        if (lint_compare(value,lint_operate (lint_new(-1), lint_new(bitcount-1), LINT_SL)) <  0) flag = 1;}
+ else { if (lint_compare(value,lint_operate (lint_new(1),  lint_new(bitcount),   LINT_SL)) >= 0) flag = 1;
+        if (lint_compare(value,lint_new(0))                                                <  0) flag = 1;}
+
+ 
+ if (!flag) {											                // flag is set if something went wrong
+        start = g_new(char,bitcount + 1);//make a new string
+        c = start + bitcount;		     //terminate it with a \0
         *c = '\0';
         while(start != c--){                                            // write the string
-            if (value & 1) *c = 'I';
-            else           *c = 'O';
-            value = value >> 1;
+            if (lint_test(value, lint_new(1))) *c = 'I';
+            else                               *c = 'O';
+            value = lint_operate(value, lint_new(1), LINT_SR);
             }
         newelement = g_new (RuleElement, 1);							// make a new rule
         newelement->string = start;
@@ -960,8 +1007,8 @@ Asm* asm_assemble_number (char* text, GList* rules, AsmStatus stat, int last)
         if (last) while (isblank(*ret->text)) ret->text++;				// if its last then walk through the rest of the spaces
         if (!last
             ||        !(*(ret->text)) 
-            || '\n' == (*(ret->text)) ) {									// unless its last and there is still chars to process then
-            return ret; 										        // return it
+            || '\n' == (*(ret->text)) ) {                               // unless its last and there is still chars to process then
+            return ret;                                                 // return it
             }
         }
   g_free(ret);        
@@ -1237,6 +1284,15 @@ GList* asm_assemble (char* text, GList* rules, AsmStatus stat, int last)
           }
           break;
 
+       case ALIGNRULE:
+          {
+          Asm* numberasm = g_new(Asm,1);
+          numberasm->text = text;
+          numberasm->binary = NULL;
+          retlist = g_list_append(retlist,numberasm);                    // add it to the list
+          }
+          break;
+
        case ASCIIRULE:
           {
           Asm* numberasm= asm_assemble_ascii (text, rules, stat, last);
@@ -1292,7 +1348,7 @@ GList* asm_copystringlist (GList* list)         // copy an asm list
             while (split){
                 if (SPLIT2FREE == ((SplitElement *)split->data)->type){
                     splitelement = g_new(SplitElement,1);
-                    splitelement->string=g_strdup(((SplitElement *)split->data)->string);
+                    splitelement->string=g_strdup(((SplitElement *)split->data)->string); //FIXME what is this? it doesn't work!
                     splitelement->type=SPLIT2FREE;
                     }
                 }
@@ -1328,13 +1384,14 @@ void asm_freestringlist (GList* list)   // free a asm list
             split = ((RuleElement *) scan->data)->split;
             while (split) {
                 if (SPLIT2FREE == ((SplitElement *)split->data)->type) {    // free all split2frees
-//                	g_list_free (((SplitElement *)split->data)->bin);         // free the list
+                    g_free (((SplitElement *)split->data)->bitfield);         // free the binary part
                     g_free (split->data);
                     }
                 split=split->next;
                 }
+		    g_list_free(((RuleElement *) scan->data)->split);
 			g_free (scan->data);                                        // and the element
-		    break;
+            break;
             
           case NONTERM2FREE:                                            // on nonterm2frees
 			asm_freestringlist (((RuleElement *) scan->data)->rules);   // free their contents
@@ -1552,7 +1609,7 @@ int asm_getsymbol_length (char* c)
 
 char asm_getchar(char* start, char** endptr)
 {
- unsigned char c;
+ unsigned char c = 0;
  int index;
  char* ptr;
   if ('\\' == *start) {
@@ -1617,3 +1674,4 @@ int asm_isset(char c)
 
 
 }
+
