@@ -77,7 +77,7 @@ void board_getcpuinfo(void)  /* called from wotRu? function */
  board_cpu_name = ScangetStringAdvance (&cpulist);
  if (VERBOSE)g_print("Detected        :%s\n",board_cpu_name);    /* Can be cut out or redirected to console*/
  board_memory_ptr_width = ScanfindSymbolNumber(cpulist,"memory-ptr-width",4);
- board_wordaline = ScanfindSymbolNumber(cpulist,"wordaline",0);
+ board_wordaline = ScanfindSymbol(cpulist,"wordaline",1,0);
  view_window_display_list = ScanfindSymbolString (cpulist,"window-list", NULL);
  view_window_display_list2 = ScanfindSymbolList (cpulist,"window-list2", NULL);
  if (view_window_display_list2) view_window_display_list2 = SCAN_LIST(SCAN_LIST_FIRST(view_window_display_list2));
@@ -630,7 +630,7 @@ void setup() /* setup function*/
      scanner = ScanOpenSCANString (dotkomodo); 	/* use internal file */
  else               scanner = ScanOpenSCANFile   (rcfile); 	/* open .komodo */
  if(!scanner) {
-        g_print("Cannot find setup file!\nIf you would like to create one then start komodo with -c flag\nOr use the default internal one with -i");
+        g_print("Cannot find setup file!\nIf you would like to create one then start komodo with -c flag\nOr use the default internal one with -i\n");
         exit(1);
         }
  Scantopnode = ScanParseSCANNode (scanner, FALSE); /* Scan .komodo file*/
@@ -644,10 +644,12 @@ void setup() /* setup function*/
         config_toplist = NULL;
         }
  else{
-        config_topnode = ScanParseSCANNode (scanner, FALSE); /* Scan .komodo file*/
+        config_topnode = ScanParseSCANNode (scanner, FALSE); /* Scan .kmd_config file*/
         ScanCloseSCANFile (scanner);
         if (SCAN_IS_LIST(config_topnode)) {
             config_toplist = SCAN_LIST(config_topnode);
+            SCAN_LIST(config_topnode)=NULL;
+            ScanDeleteSCANNode(config_topnode);
             }
         else config_toplist = NULL;
         }
