@@ -15,7 +15,7 @@ uchar* view_int2chararr(int,int);
 void view_chararrCpychararr(int, uchar* , uchar*);
 int view_hexstr2chararr2(int, char*, uchar*);
 int view_hexstr2chararr(int, char*, uchar*);
-int view_findstrinarr(char*, char**,int);
+int view_findstrinarr(char*, GList**,int);
 char* view_chararrinregbank(int,uchar*);
 void g_list_free_contents (GList* list);
 
