@@ -37,6 +37,29 @@ GdkColor view_greycolour = {1,0xc000,0xc000,0xa000};
 GdkColor view_redcolour  = {1,0xFFFF,0x0000,0x0000};
 GdkColor view_blackcolour ={1,0x0000,0x0000,0x0000};
 
+mem_window_length window_lengths[] = {
+// "NAME",          unit size, line size,  in simple mode, initial
+    {"ISA",                 0, 4, 0},
+    {"Byte",                1, 1, 0},    
+#ifndef SIMPLE_INTERFACE
+    {"Byte Byte",           1, 2, 0},    
+    {"Byte Byte Byte Byte", 1, 4, 0},    
+#endif
+    {"Half",                2, 2, 0},    
+#ifndef SIMPLE_INTERFACE
+    {"Half Half",           2, 4, 0},    
+    {"Half Half Half Half", 2, 8, 0},    
+#endif
+    {"Word",                4, 4, 1},    
+#ifndef SIMPLE_INTERFACE
+    {"Word Word",           4, 8, 0},    
+    {"Double",              8, 8, 0},
+#endif
+    {NULL,                  0, 0, 0}    };
+    
+
+
+
 /*              These are ARM.C disassemble instructions replaced by LITHP in ASM.C
  * 
  * char* view_dis_arm(uchar* address, uchar* code) // disassemble an arm instruction
@@ -60,11 +83,10 @@ GdkColor view_blackcolour ={1,0x0000,0x0000,0x0000};
  * }
  */
 
-char* view_dis(uchar* address, uchar* code,int count, GList* isa) /* disassemble an instruction count specifies the length*/
+char* view_dis(uchar* address, uchar* code, int count, GList* isa, int* consumed) /* disassemble an instruction count specifies the length*/
 {
  Disasm disasm;
- char* ret = g_strdup("");
- char* temp;
+ char* ret;
  char* codelist;
  int intaddress = view_chararr2int(board_memory_ptr_width, address);
  AsmStatus stat = asm_default_status;
@@ -73,22 +95,30 @@ char* view_dis(uchar* address, uchar* code,int count, GList* isa) /* disassemble
  disasm.bitfield= codelist;
 // do     {
         stat.address = intaddress;
+//   g_print("characters:%d \n", count);
+//   g_print("testb\n");
         disasm = asm_disassemble (disasm.bitfield, isa, stat);
+//   g_print("testb2\n");
         
           if (disasm.distext == &dis_error ) {
-                            temp = g_strconcat (ret, "Undefined Instruction", NULL);
-                            g_free (ret);
-                            ret=temp;
+                            ret = g_strdup("Undefined Instruction");
                             disasm.bitfield=NULL;
-                             } 
-            else                    {
-                            ret = asm_sprint(disasm.distext, ret);
+//                            g_print("assembled Undef");
+                            if (consumed) *consumed = 1;
+                            } 
+            else {
+                            ret = asm_sprint(disasm.distext, g_strdup(""));
                             asm_freestringlist (disasm.distext);
+//                            g_print("assembled:%d from:%d, %s\n", (disasm.bitfield-codelist), count, ret);
+                            
+                            if (consumed) *consumed = (disasm.bitfield - codelist)/8;
                             }
                             
+//   g_print("testb\n");
                             
  //        } while (disasm.bitfield);
-         
+ 
+ 
  g_free(codelist);
  
  
@@ -143,84 +173,116 @@ char* view_dis(uchar* address, uchar* code,int count, GList* isa) /* disassemble
 int view_updatememwindow(mem_window* memwindowptr)   /* update memory window*/
 {
  GtkCList* clist = GTK_CLIST(memwindowptr->clist_ptr);
- int count = memwindowptr->count * memwindowptr->width;
+ int bytesperinstruction = 8;
+ int count = memwindowptr->count * (memwindowptr->gran ? memwindowptr->width : bytesperinstruction);
  unsigned char* memdata = g_new(unsigned char, count);
  char* data;
+ char* dis_data;
  char* address;
  int temp;
  int flag;
  char* test;
  char* test2;
+ int offset=0;
  GList* breaks=NULL;
  GList* breaksinactive=NULL;
  GList* templist;
- GdkColor* colour;
+ GdkColor* colour = NULL;
+ int consumed;
  uchar* new_address = view_chararrAddchararr(board_memory_ptr_width,
                                memwindowptr->address, memwindowptr->regoffset);
-  {
-    unsigned int worda;
-    unsigned int wordb;
-    int temp, temp2;
-    uchar* value;
-    uchar repc;
-    board_sendchar(BR_BP_GET);
-    if (4 != board_getb32(&worda)) {g_free(new_address);return 0;}    //get breaks and breaksinactive list
-    if (4 != board_getb32(&wordb)) {g_free(new_address);return 0;}
-    for (temp=0;temp<32;temp++)
-        if ((worda>>temp)&1){
- //if (VERBOSE) g_print("foundnumber:%d",temp);
-            value=g_new(uchar,board_memory_ptr_width);
-            board_sendchar(BR_BP_READ);
-            board_sendchar(temp);
-            flag=0;
-            for (temp2=2;temp2;temp2--){
-                if (1 != board_getchar(&repc)) {g_free(new_address);return 0;}
-                if (0xff != repc) flag=1;
-                }
-            if (board_memory_ptr_width != board_getchararray(board_memory_ptr_width,value)) 
-                                               {g_free(new_address);return 0;}
- //if (VERBOSE) g_print("found:%x\n",view_chararr2int(board_memory_ptr_width,value));
-            for (temp2=board_memory_ptr_width;temp2;temp2--){
-                if (1 != board_getchar(&repc)) {g_free(new_address);return 0;}
-                if (0xff != repc) flag=1;
-                }
-            for (temp2=16;temp2;temp2--){
-                if (1 != board_getchar(&repc)) {g_free(new_address);return 0;}
-                if (0x0 != repc) flag=1;
-                }
-            if (flag) g_free(value);
-            else    if ((wordb>>temp)&1) breaks = g_list_append (breaks, value);
-                    else breaksinactive = g_list_append (breaksinactive, value);
-            }
-  }
- if (board_wordaline) new_address[0] &= -memwindowptr->width;
- if (!board_get_memory (count / memwindowptr->gran, new_address, memdata, memwindowptr->gran)){
+ {
+   unsigned int worda;
+   unsigned int wordb;
+   int temp, temp2;
+   uchar* value;
+   uchar repc;
+   board_sendchar(BR_BP_GET);
+   if (4 != board_getb32(&worda)) {g_free(new_address);return 0;}    //get breaks and breaksinactive list
+   if (4 != board_getb32(&wordb)) {g_free(new_address);return 0;}
+
+   for (temp=0;temp<32;temp++)
+       if ((worda>>temp)&1){
+//if (VERBOSE) g_print("foundnumber:%d",temp);
+           value=g_new(uchar,board_memory_ptr_width);
+           board_sendchar(BR_BP_READ);
+           board_sendchar(temp);
+           flag=0;
+           for (temp2=2;temp2;temp2--){
+               if (1 != board_getchar(&repc)) {g_free(new_address);return 0;}
+               if (0xff != repc) flag=1;
+               }
+           if (board_memory_ptr_width != board_getchararray(board_memory_ptr_width,value)) 
+                                              {g_free(new_address);return 0;}
+//if (VERBOSE) g_print("found:%x\n",view_chararr2int(board_memory_ptr_width,value));
+           for (temp2=board_memory_ptr_width;temp2;temp2--){
+               if (1 != board_getchar(&repc)) {g_free(new_address);return 0;}
+               if (0xff != repc) flag=1;
+               }
+           for (temp2=16;temp2;temp2--){
+               if (1 != board_getchar(&repc)) {g_free(new_address);return 0;}
+               if (0x0 != repc) flag=1;
+               }
+           if (flag) g_free(value);
+           else    if ((wordb>>temp)&1) breaks = g_list_append (breaks, value);
+                   else breaksinactive = g_list_append (breaksinactive, value);
+           }
+ }
+
+ if (board_wordaline && memwindowptr->gran) new_address[0] &= -memwindowptr->width;
+ if (!board_get_memory (memwindowptr->gran ? (count / memwindowptr->gran) : count, new_address, memdata, memwindowptr->gran ? (memwindowptr->gran) : 1)){
    if (!board_mini_ping(0) ||
-       !board_get_memory (count / memwindowptr->gran, new_address, memdata, memwindowptr->gran)){       /*get new memory data*/
+       !board_get_memory (memwindowptr->gran ? (count / memwindowptr->gran) : count, new_address, memdata, memwindowptr->gran ? (memwindowptr->gran) : 1)){       /*get new memory data*/
                               g_free(new_address);g_free(memdata); return 0;}
    }
+
  gtk_clist_freeze (clist); 
- count = 0;
- while (count < memwindowptr->count){
+ 
+ for (count=0;count < memwindowptr->count; count++){                            // For all lines
    address = view_chararrAddint(board_memory_ptr_width,
-                       new_address, count * memwindowptr->width);
+                       new_address, offset);
+//   g_print("testA\n");
+   if ((!memwindowptr->gran) && (memwindowptr->count*bytesperinstruction-offset < 20)){
+//        g_print("%d - %d\n", memwindowptr->count*bytesperinstruction, offset);
+        memdata = g_realloc(memdata, sizeof(unsigned char) * memwindowptr->count * (bytesperinstruction+1));
+//        g_print("testB\n");
+        if (!board_get_memory (memwindowptr->count, new_address, memdata+(memwindowptr->count*bytesperinstruction ), 1)){
+          if (!board_mini_ping(0) ||
+              !board_get_memory (memwindowptr->count, new_address, memdata+(memwindowptr->count*bytesperinstruction), 1)){       /*get more memory data*/
+                                     g_free(new_address);g_free(memdata); return 0;}
+          }
+        bytesperinstruction++;
+        }
+//   g_print("testC\n");
+
+   dis_data = view_dis(address, memdata + offset,
+                          memwindowptr->gran ? (memwindowptr->width) : (memwindowptr->count*bytesperinstruction-offset),
+                          memwindowptr->isa, &consumed);
+
+//   g_print("testa\n");
    test = view_chararrinregbank (board_memory_ptr_width, address);
 
+//   g_print("testa\n");
    data = view_chararr2hexstrbe (board_memory_ptr_width, address);              //create an address string
+   memwindowptr->offsets[count] = offset;
     
+//   g_print("testa\n");
    
    {
-    Symbol* symbol = symbol_findaddress(symbol_table, lint_new(view_chararr2long(board_memory_ptr_width, address)));
+    Symbol* symbol = symbol_findaddress(symbol_table, lint_from_chararr(board_memory_ptr_width, address));
     if (symbol){
         g_free(data);
         data = g_strdup(symbol->name);
 //        g_print("Printing symbol:\"%s\" %x\n", data, view_chararr2long(board_memory_ptr_width, address));
-        }
+        }                                                                       // search for symbol
    }
    
+//   g_print("testa\n");
    
    if (test) {
-        test2 = g_strconcat (data, " " , test, NULL);
+//   g_print("testb\n");
+        test2 = g_strconcat (data, " " , test, NULL);                           // append reg to address/symbol
+//   g_print("testc\n");
         g_free(data);
         g_free(test);
         data = test2;
@@ -228,7 +290,9 @@ int view_updatememwindow(mem_window* memwindowptr)   /* update memory window*/
 
    flag=0;
 
-   templist=breaksinactive;
+//   g_print("testa\n");
+
+   templist=breaksinactive;                                                     // search inactive breakpoints
    while(templist){
         if (!view_chararrSubchararr(board_memory_ptr_width, address, templist->data)) {
             gtk_clist_set_pixtext(clist,count,0,data,0,view_break_inactive_pixmap,view_break_inactive_bitmap);
@@ -236,7 +300,7 @@ int view_updatememwindow(mem_window* memwindowptr)   /* update memory window*/
             }
         templist=templist->next;
         }
-   templist=breaks;
+   templist=breaks;                                                     // search active breakpoints
    while(templist){
         if (!view_chararrSubchararr(board_memory_ptr_width, address, templist->data)) {
             flag=2;
@@ -245,6 +309,8 @@ int view_updatememwindow(mem_window* memwindowptr)   /* update memory window*/
          templist=templist->next;
         }
 
+
+
    for (temp=0;temp<special_register_count;temp++)
         if (special_registers[temp].active)
             if (!view_chararrSubchararr(board_memory_ptr_width, special_registers[temp].value,address)){
@@ -261,10 +327,11 @@ int view_updatememwindow(mem_window* memwindowptr)   /* update memory window*/
    g_free (data);
 
 
-   for (temp=0;temp<(memwindowptr->width/memwindowptr->gran);temp++){  /*insert new info into window*/
-     data = view_chararr2hexstrbe (memwindowptr->gran, memdata  +
-                                    count * memwindowptr->width +
+   for (temp=0;temp<(memwindowptr->gran ? (memwindowptr->width/memwindowptr->gran) : 1);temp++){  // insert new hex info into window
+     data = view_chararr2hexstrbe (memwindowptr->gran ? memwindowptr->gran : consumed, memdata  +
+                                    offset +
                                     temp  * memwindowptr->gran);
+//     g_print("data:%s\n",data);
      if (!gtk_clist_get_text(clist,count,temp+1,&test)) gtk_clist_set_text(clist,count,temp+1,data);
      else if (strcmp(data,test)) gtk_clist_set_text(clist,count,temp+1,data);
      g_free (data);
@@ -286,36 +353,35 @@ int view_updatememwindow(mem_window* memwindowptr)   /* update memory window*/
         }
 
 
-   data = view_chararr2asciistr (memwindowptr->width, memdata + count
-                               * memwindowptr->width);
-   if (!gtk_clist_get_text(clist,count,5,&test)) gtk_clist_set_text(clist,count,5,data);
+   data = view_chararr2asciistr (memwindowptr->gran ? memwindowptr->width : consumed, memdata + offset);
+   if (!gtk_clist_get_text(clist,count,5,&test)) gtk_clist_set_text(clist,count,5,data);    // print ascii
    else if (strcmp(data,test)) gtk_clist_set_text(clist,count,5,data);
    g_free (data);
-   data = view_dis(address, memdata + count * memwindowptr->width,
-                         memwindowptr->width, memwindowptr->isa);
-   if (!gtk_clist_get_text(clist,count,6,&test)) gtk_clist_set_text(clist,count,6,data);
-   else if (strcmp(data,test)) gtk_clist_set_text(clist,count,6,data);
+   
+   if (!gtk_clist_get_text(clist,count,6,&test)) gtk_clist_set_text(clist,count,6,dis_data);    // print disassembly
+   else if (strcmp(dis_data,test)) gtk_clist_set_text(clist,count,6,dis_data);  
    
     
 
-   g_free (data);
+   g_free (dis_data);
    g_free (address);
-   count++;
+   if (memwindowptr->gran) 
+        offset += memwindowptr->width;
+   else offset += consumed;
+        
    }
-   g_free (new_address);
-   g_free(memdata);
- templist=breaksinactive;
- while(templist){
-        g_free(templist->data);
-        templist=templist->next;
-        }
- templist=breaks;
- while(templist){
-        g_free(templist->data);
-        templist=templist->next;
-        }
- 
- 
+   
+   
+   
+   
+  g_free (new_address);
+  g_free(memdata);
+
+  g_list_free_contents(breaksinactive);
+  g_list_free(breaksinactive);
+  g_list_free_contents(breaks);
+  g_list_free(breaks);
+   
   gtk_clist_thaw (clist);
   return 1;
 }
@@ -334,7 +400,7 @@ int view_updateregwindow(reg_window* regwindowptr) /* refresh register window*/
  gtk_clist_freeze (clist);
  count = 0;
  while (count < regbank->number){ 
-   data[0] = g_strdup(regbank->names[count]);
+   data[0] = g_strdup(regbank->names[count]->data);
    if (regbank->width) data[1] = view_chararr2hexstrbe (regbank->width,
                                &regbank->values[count * regbank->width]);
    else {if (regbank->values[count]) data[1]="1"; else data[1]="0";}
@@ -466,7 +532,7 @@ GtkWidget* view_create_filebar (void)
   GtkWidget *buttoncancel;
   GtkWidget *optionmenu_isa;
   GtkWidget *optionmenu_menu;
-  GtkWidget *menuitem;
+  GtkWidget *menuitem = NULL;
   DefinitionStack* isa_list;
 
   vbox = gtk_vbox_new (FALSE, 2);
@@ -1279,6 +1345,9 @@ GtkWidget* create_splashwindow (void)
   gtk_container_add (GTK_CONTAINER (view_splashwindow), vbox);
 
   pixmap = gtk_pixmap_new  (view_komodologo_pixmap, view_komodologo_bitmap);
+  gdk_pixmap_unref  (view_komodologo_pixmap);
+  gdk_bitmap_unref (view_komodologo_bitmap);
+  
   gtk_widget_show (pixmap);
   gtk_box_pack_start (GTK_BOX (vbox), pixmap, FALSE, FALSE, 0);
   gtk_widget_ref (view_splashwindow);
@@ -1917,7 +1986,7 @@ GtkWidget* view_create_memory_clist (void)
   mem_window_list **memwindowlist = &view_memwindowlist;
   
   while (NULL != *memwindowlist) memwindowlist = &((*memwindowlist)->next);
-  *memwindowlist = g_new(mem_window_list,1);
+  *memwindowlist = g_new(mem_window_list,1);                                    //FIXME turn memwindowlist into GLIist
   (*memwindowlist)->mem_data_ptr = memwindow;
   (*memwindowlist)->next = NULL;
   
@@ -1930,13 +1999,12 @@ GtkWidget* view_create_memory_clist (void)
   gtk_widget_show (vbox);
   gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
   
-  memwindow->address= g_new0(uchar,4);
-  memwindow->regoffset = board_nulreg;
-  memwindow->count=46;
+  memwindow->address= g_new0(uchar,4);          //FIXME pointer size?
+  memwindow->regoffset = board_nulreg;          
+  memwindow->count=10;
+  memwindow->offsets=g_new0(int, 10);
   memwindow->width=4;
   memwindow->gran=4;
-//  memwindow->wait=100;
-//  memwindow->timer=0;
   
   vscale_adj = gtk_adjustment_new (0, -3.25, 3.25, 3, 3, 0);
   vscale = gtk_vscale_new (GTK_ADJUSTMENT (vscale_adj));
@@ -1982,84 +2050,31 @@ for (temp=0;temp<7;temp++){
   optionmenu_menu = gtk_menu_new();
   gtk_widget_ref (optionmenu_menu);
 
-  menuitem = gtk_menu_item_new_with_label (_("Byte"));
-  gtk_widget_show (menuitem);
-  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
-  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
-                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)0x11);
-
-#ifndef SIMPLE_INTERFACE
-  menuitem = gtk_menu_item_new_with_label (_("Byte Byte"));
-  gtk_widget_show (menuitem);
-  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
-  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
-                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)0x12);
-#endif
-
-#ifndef SIMPLE_INTERFACE
-  menuitem = gtk_menu_item_new_with_label ("Byte Byte Byte Byte");
-  gtk_widget_show (menuitem);
-  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
-  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
-                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)0x14);
-#endif
-
-  menuitem = gtk_menu_item_new_with_label (_("Half"));
-  gtk_widget_show (menuitem);
-  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
-  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
-                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)0x22);
-
-
-#ifndef SIMPLE_INTERFACE
-  menuitem = gtk_menu_item_new_with_label (_("Half Half"));
-  gtk_widget_show (menuitem);
-  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
-  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
-                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)0x24);
-#endif
-
-#ifndef SIMPLE_INTERFACE
-  menuitem = gtk_menu_item_new_with_label (_("Half Half Half Half"));
-  gtk_widget_show (menuitem);
-  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
-  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
-                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)0x28);
-#endif
-
-  menuitem = gtk_menu_item_new_with_label (_("Word"));
-  gtk_widget_show (menuitem);
-  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
-  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
-                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)0x44);
-
-#ifndef SIMPLE_INTERFACE
-  menuitem = gtk_menu_item_new_with_label ("Word Word");
-  gtk_widget_show (menuitem);
-  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
-  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
-                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)0x48);
-#endif
-
-  menuitem = gtk_menu_item_new_with_label ("Double");
-  gtk_widget_show (menuitem);
-  gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
-  gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
-                   GTK_SIGNAL_FUNC (callback_memwindow_length),(gpointer)0x88);
-
+  int index; 
+  int got_initial_window_length=0;
+  int initial_window_length=0; 
+  int option_list_count=0; 
+  
+  for (index=0; window_lengths[index].name; index++) {
+    menuitem = gtk_menu_item_new_with_label (_(window_lengths[index].name));
+    gtk_widget_show (menuitem);
+    gtk_menu_append (GTK_MENU (optionmenu_menu), menuitem);
+    gtk_signal_connect (GTK_OBJECT(menuitem), "activate",
+                     GTK_SIGNAL_FUNC (callback_memwindow_length),&(window_lengths[index]));
+    if (window_lengths[index].initial && !got_initial_window_length){
+        got_initial_window_length=1;
+        initial_window_length=index;
+        }
+    if (!got_initial_window_length) option_list_count++;
+    }
 
 
 
   gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu_length),
                                              optionmenu_menu);
                                              
-#ifdef SIMPLE_INTERFACE
-  gtk_option_menu_set_history     (GTK_OPTION_MENU (optionmenu_length), 2);
-#endif
+  gtk_option_menu_set_history (GTK_OPTION_MENU (optionmenu_length), option_list_count);
 
-#ifndef SIMPLE_INTERFACE
-  gtk_option_menu_set_history     (GTK_OPTION_MENU (optionmenu_length), 6);
-#endif
 
   gtk_box_pack_start (GTK_BOX (hbox1), optionmenu_length, TRUE, TRUE, 0);
   gtk_object_set_data (GTK_OBJECT (optionmenu_menu), "mem_window", memwindow);
@@ -2097,7 +2112,7 @@ for (temp=0;temp<7;temp++){
 
 
 #ifndef SIMPLE_INTERFACE
-  listsize_adj = gtk_adjustment_new (46, 0, 40, 1, 1, 1);
+  listsize_adj = gtk_adjustment_new (10, 0, 40, 1, 1, 1);
   listsize = gtk_spin_button_new (GTK_ADJUSTMENT (listsize_adj), 1, 0);
   gtk_widget_ref (listsize);
   gtk_widget_show (listsize);
@@ -2226,7 +2241,7 @@ for (temp=0;temp<7;temp++){
   gtk_clist_set_column_widget (GTK_CLIST (clist), 6, label);  
   
   gtk_widget_set_style(clist,fixed_style);
-  callback_memwindow_length (GTK_MENU_ITEM(menuitem), (gpointer)0x44);
+  callback_memwindow_length (GTK_MENU_ITEM(menuitem), &(window_lengths[initial_window_length]));
 //  view_updatememwindow(memwindow); 
   return hbox;
 }
